from .lexer import Lexer

__version__ = "0.0.4"

__title__ = "streamingjson"
__description__ = (
    "A streamlined, user-friendly JSON streaming preprocessor, crafted in Python."
)
__url__ = "https://github.com/karminski/streaming-json-py"
__uri__ = __url__
__doc__ = f"{__description__} <{__uri__}>"

__author__ = "Karminski"
__email__ = "code.karminski@outlook.com"

__license__ = "MIT"
__copyright__ = "Copyright 2024 Karminski"

__all__ = [
    "Lexer",
]
